/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsSaleRevenue.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.unis.financialSupervision.modular.priceApproval.entity.DataCPZ;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.unis.common.annotation.CommonLog;
import com.unis.common.pojo.CommonResult;
import com.unis.financialSupervision.modular.fsSaleRevenue.entity.FsSaleRevenue;
import com.unis.financialSupervision.modular.fsSaleRevenue.param.FsSaleRevenueIdParam;
import com.unis.financialSupervision.modular.fsSaleRevenue.param.FsSaleRevenuePageParam;
import com.unis.financialSupervision.modular.fsSaleRevenue.service.FsSaleRevenueService;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * 销售收入上缴控制器
 *
 * @author QSHan
 * @date 2024/07/29 14:04
 */
@Api(tags = "销售收入上缴控制器")
@ApiSupport(author = "UNIS_TEAM", order = 1)
@RestController
@Validated
public class FsSaleRevenueController {

    @Resource
    private FsSaleRevenueService fsSaleRevenueService;

    /**
     * 获取销售收入上缴分页
     *
     * @author QSHan
     * @date 2024/07/29 14:04
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取销售收入上缴分页")
//    @SaCheckPermission("/financialSupervision/fsSaleRevenue/page")
    @GetMapping("/financialSupervision/fsSaleRevenue/page")
    public CommonResult<Page<FsSaleRevenue>> page(FsSaleRevenuePageParam fsSaleRevenuePageParam) {
        return CommonResult.data(fsSaleRevenueService.page(fsSaleRevenuePageParam));
    }

    /**
     * 编辑销售收入上缴
     *
     * @author QSHan
     * @date 2024/07/29 14:04
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑销售收入上缴")
    @CommonLog("编辑销售收入上缴")
//    @SaCheckPermission("/financialSupervision/fsSaleRevenue/edit")
    @PostMapping("/financialSupervision/fsSaleRevenue/edit")
    public CommonResult<String> edit(@RequestBody @Valid FsSaleRevenue param) {
        fsSaleRevenueService.edit(param);
        return CommonResult.ok();
    }

    /**
     * 获取销售收入上缴详情
     *
     * @author QSHan
     * @date 2024/07/29 14:04
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取销售收入上缴详情")
//    @SaCheckPermission("/financialSupervision/fsSaleRevenue/detail")
    @GetMapping("/financialSupervision/fsSaleRevenue/detail")
    public CommonResult<FsSaleRevenue> detail(@Valid FsSaleRevenueIdParam fsSaleRevenueIdParam) {
        return CommonResult.data(fsSaleRevenueService.detail(fsSaleRevenueIdParam));
    }

    @GetMapping("/financialSupervision/fsSaleRevenue/getOutStorageDetailed/{contractNumber}")
    @ApiOperation(value = "三账一致性-出库明细", notes = "出库明细")
    public CommonResult<List<DataCPZ>> getOutStorageDetailed(@PathVariable("contractNumber") String contractNumber) {
        return CommonResult.data(fsSaleRevenueService.getOutStorageDetailed(contractNumber));
    }
}
