/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsSaleRevenue.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.common.constant.DBConstant;
import com.unis.financialSupervision.modular.fsSaleRevenue.entity.BusinessContractDetail;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.entity.FsSaleRevenueApprovalPrice;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.mapper.FsSaleRevenueApprovalPriceMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.entity.FsSaleRevenueDiffPrice;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.mapper.FsSaleRevenueDiffPriceMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.entity.FsSaleRevenueInspectionLoss;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.mapper.FsSaleRevenueInspectionLossMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.entity.FsSaleRevenueSettle;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.mapper.FsSaleRevenueSettleMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueThreeConsistence.entity.FsSaleRevenueThreeConsistence;
import com.unis.financialSupervision.modular.fsSaleRevenueThreeConsistence.mapper.FsSaleRevenueThreeConsistenceMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueThreeConsistenceAccount.entity.FsSaleRevenueThreeConsistenceAccount;
import com.unis.financialSupervision.modular.fsSaleRevenueThreeConsistenceAccount.mapper.FsSaleRevenueThreeConsistenceAccountMapper;
import com.unis.financialSupervision.modular.priceApproval.entity.DataCPZ;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;
import com.unis.financialSupervision.modular.fsSaleRevenue.entity.FsSaleRevenue;
import com.unis.financialSupervision.modular.fsSaleRevenue.mapper.FsSaleRevenueMapper;
import com.unis.financialSupervision.modular.fsSaleRevenue.param.FsSaleRevenueAddParam;
import com.unis.financialSupervision.modular.fsSaleRevenue.param.FsSaleRevenueEditParam;
import com.unis.financialSupervision.modular.fsSaleRevenue.param.FsSaleRevenueIdParam;
import com.unis.financialSupervision.modular.fsSaleRevenue.param.FsSaleRevenuePageParam;
import com.unis.financialSupervision.modular.fsSaleRevenue.service.FsSaleRevenueService;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 销售收入上缴Service接口实现类
 *
 * @author QSHan
 * @date  2024/07/29 14:04
 **/
@Service
@DS(DBConstant.DS_DEPOT)
public class FsSaleRevenueServiceImpl extends ServiceImpl<FsSaleRevenueMapper, FsSaleRevenue> implements FsSaleRevenueService {
    @Resource
    private FsSaleRevenueMapper saleRevenueMapper;
    @Resource
    private FsSaleRevenueInspectionLossMapper saleRevenueInspectionLossMapper;
    @Resource
    private FsSaleRevenueThreeConsistenceAccountMapper threeConsistenceAccountMapper;
    @Resource
    private FsSaleRevenueThreeConsistenceMapper threeConsistenceMapper;
    @Resource
    private FsSaleRevenueSettleMapper settleMapper;
    @Resource
    private FsSaleRevenueApprovalPriceMapper approvalPriceMapper;
    @Resource
    private FsSaleRevenueDiffPriceMapper diffPriceMapper;

    @Override
    public Page<FsSaleRevenue> page(FsSaleRevenuePageParam fsSaleRevenuePageParam) {
        QueryWrapper<FsSaleRevenue> queryWrapper = new QueryWrapper<>();
        queryWrapper.ne("sr.`status`", 0);
        if (StringUtils.isNotBlank(fsSaleRevenuePageParam.getDwmc())) {
            queryWrapper.like("ui.dwmc", fsSaleRevenuePageParam.getDwmc());
        }
        if (StringUtils.isNotBlank(fsSaleRevenuePageParam.getOrgName())) {
            queryWrapper.like("oi.org_name", fsSaleRevenuePageParam.getOrgName());
        }

        if(ObjectUtil.isAllNotEmpty(fsSaleRevenuePageParam.getSortField(), fsSaleRevenuePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(fsSaleRevenuePageParam.getSortOrder());
            queryWrapper.orderBy(true, fsSaleRevenuePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(fsSaleRevenuePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByDesc(FsSaleRevenue::getId);
        }
        return saleRevenueMapper.pageList(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FsSaleRevenue param) {
        // 修改主表
        this.updateById(param);

        // 修改子表-质检损益信息
        this.inspectionLossUpdate(param.getInspectionLossList());

        // 修改子表-三账一致性-会计账
        this.threeConsistenceAccountUpdate(param.getThreeConsistenceAccountList());

        // 修改子表-三账一致性-实际出库情况
        this.threeConsistenceUpdate(param.getThreeConsistenceList());

        // 修改子表-结算信息
        this.revenueSettleUpdate(param.getRevenueSettleList());

        // 修改子表-原核定入库价格
        this.approvalPriceUpdate(param.getApprovalPriceList());

        // 修改子表-差价上缴
        this.diffPriceUpdate(param.getDiffPriceList());
    }

    @Override
    public FsSaleRevenue detail(FsSaleRevenueIdParam fsSaleRevenueIdParam) {
        // 查询主表
        FsSaleRevenue saleRevenue = saleRevenueMapper.queryEntityById(fsSaleRevenueIdParam.getId());

        // 查询子表-合同信息
        List<BusinessContractDetail> contractInfoList = saleRevenueMapper.getContractInfoList(saleRevenue.getContractId());
        if (CollectionUtils.isNotEmpty(contractInfoList)) {
            saleRevenue.setContractDetails(contractInfoList);
        }
        // 查询子表-质检损益信息
        List<FsSaleRevenueInspectionLoss> inspectionLossList = saleRevenueMapper.getQualityDetailsInfo(fsSaleRevenueIdParam.getId());
        if (CollectionUtils.isNotEmpty(inspectionLossList)) {
            saleRevenue.setInspectionLossList(inspectionLossList);
        }
        // 查询子表-三账一致性-会计账
        List<FsSaleRevenueThreeConsistenceAccount> threeConsistenceList = saleRevenueMapper.getConsistenceAccountDetailsInfo(fsSaleRevenueIdParam.getId());
        if (CollectionUtils.isNotEmpty(threeConsistenceList)) {
            saleRevenue.setThreeConsistenceAccountList(threeConsistenceList);
        }
        // 查询子表-三账一致性-实际出库情况
        List<FsSaleRevenueThreeConsistence> outStorageDetailsInfo = saleRevenueMapper.getOutStorageDetailsInfo(fsSaleRevenueIdParam.getId());
        if (CollectionUtils.isNotEmpty(outStorageDetailsInfo)) {
            saleRevenue.setThreeConsistenceList(outStorageDetailsInfo);
        }
        // 查询子表-结算信息
        List<FsSaleRevenueSettle> revenueSettleList = saleRevenueMapper.getSettlementDetailsInfo(fsSaleRevenueIdParam.getId());
        if (CollectionUtils.isNotEmpty(revenueSettleList)) {
            saleRevenue.setRevenueSettleList(revenueSettleList);
        }
        // 查询子表-原核定入库价格
        List<FsSaleRevenueApprovalPrice> approvalPriceList = saleRevenueMapper.getCardDetailsInfo(fsSaleRevenueIdParam.getId());
        if (CollectionUtils.isNotEmpty(approvalPriceList)) {
            saleRevenue.setApprovalPriceList(approvalPriceList);
        }
        // 查询子表-差价上缴
        List<FsSaleRevenueDiffPrice> DiffPriceList = saleRevenueMapper.getDiffPriceDetailsInfo(fsSaleRevenueIdParam.getId());
        if (CollectionUtils.isNotEmpty(DiffPriceList)) {
            saleRevenue.setDiffPriceList(DiffPriceList);
        }

        return saleRevenue;
    }

    @Override
    public List<DataCPZ> getOutStorageDetailed(String contractNumber) {
        return saleRevenueMapper.getOutStorageDetailed(contractNumber);
    }

    // 修改子表-质检损益信息
    private void inspectionLossUpdate(List<FsSaleRevenueInspectionLoss> inspectionLossList) {
        if (CollectionUtils.isNotEmpty(inspectionLossList)) {
            inspectionLossList.forEach(inspectionLoss -> saleRevenueInspectionLossMapper.updateById(inspectionLoss));
        }
    }

    // 修改子表-三账一致性-会计账
    private void threeConsistenceAccountUpdate(List<FsSaleRevenueThreeConsistenceAccount> threeConsistenceAccountList) {
        if (CollectionUtils.isNotEmpty(threeConsistenceAccountList)) {
            threeConsistenceAccountList.forEach(threeConsistenceAccount -> threeConsistenceAccountMapper.updateById(threeConsistenceAccount));
        }
    }

    // 修改子表-三账一致性-实际出库情况
    private void threeConsistenceUpdate(List<FsSaleRevenueThreeConsistence> threeConsistenceList) {
        if (CollectionUtils.isNotEmpty(threeConsistenceList)) {
            threeConsistenceList.forEach(threeConsistence -> threeConsistenceMapper.updateById(threeConsistence));
        }
    }

    // 修改子表-结算信息
    private void revenueSettleUpdate(List<FsSaleRevenueSettle> revenueSettleList) {
        if (CollectionUtils.isNotEmpty(revenueSettleList)) {
            revenueSettleList.forEach(revenueSettle -> settleMapper.updateById(revenueSettle));
        }
    }

    // 修改子表-原核定入库价格
    private void approvalPriceUpdate(List<FsSaleRevenueApprovalPrice> approvalPriceList) {
        if (CollectionUtils.isNotEmpty(approvalPriceList)) {
            approvalPriceList.forEach(approvalPrice -> approvalPriceMapper.updateById(approvalPrice));
        }
    }

    // 修改子表-差价上缴
    private void diffPriceUpdate(List<FsSaleRevenueDiffPrice> DiffPriceList) {
        if (CollectionUtils.isNotEmpty(DiffPriceList)) {
            DiffPriceList.forEach(DiffPrice -> diffPriceMapper.updateById(DiffPrice));
        }
    }
}
